@@ -36,7 +36,7 @@ import logging.config
 import urlparse
 from boto.exception import InvalidUriError
 
-__version__ = '2.9.6'
+__version__ = '2.10.0'
 Version = __version__  # for backware compatibility
 
 UserAgent = 'Boto/%s (%s)' % (__version__, sys.platform)
@@ -765,18 +765,6 @@ def storage_uri(uri_str, default_scheme='file', debug=0, validate=True,
     # (the latter includes an optional host/net location part).
     end_scheme_idx = uri_str.find('://')
     if end_scheme_idx == -1:
-        # Check for common error: user specifies gs:bucket instead
-        # of gs://bucket. Some URI parsers allow this, but it can cause
-        # confusion for callers, so we don't.
-        colon_pos = uri_str.find(':')
-        if colon_pos != -1:
-            # Allow Windows path names including drive letter (C: etc.)
-            drive_char = uri_str[0].lower()
-            if not (platform.system().lower().startswith('windows')
-                    and colon_pos == 1
-                    and drive_char >= 'a' and drive_char <= 'z'):
-              raise InvalidUriError('"%s" contains ":" instead of "://"' %
-                                    uri_str)
         scheme = default_scheme.lower()
         path = uri_str
     else:
