@@ -72,9 +72,14 @@ def connect_to_region(region_name, **kw_params):
     :return: A connection to the given region, or None if an invalid region
              name is given
     """
+    if 'region' in kw_params and isinstance(kw_params['region'], RegionInfo)\
+       and region_name == kw_params['region'].name:
+        return EC2Connection(**kw_params)
+
     for region in regions(**kw_params):
         if region.name == region_name:
             return region.connect(**kw_params)
+
     return None
 
 
