@@ -22,15 +22,17 @@
 """
 High-level abstraction of an EC2 server
 """
-import boto, boto.utils
+import boto
+import boto.utils
 from boto.mashups.iobject import IObject
 from boto.pyami.config import Config, BotoConfigPath
 from boto.mashups.interactive import interactive_shell
-from boto.sdb.persist.object import SDBObject
-from boto.sdb.persist.property import *
+from boto.sdb.db.model import Model
+from boto.sdb.db.property import StringProperty
 import os
 import StringIO
 
+
 class ServerSet(list):
 
     def __getattr__(self, name):
@@ -55,9 +57,13 @@ class ServerSet(list):
             results.append(fn(*args))
         return results
 
-class Server(SDBObject):
+class Server(Model):
 
-    ec2 = boto.connect_ec2()
+    @property
+    def ec2(self):
+        if self._ec2 is None:
+            self._ec2 = boto.connect_ec2()
+        return self._ec2
 
     @classmethod
     def Inventory(cls):
@@ -80,24 +86,26 @@ class Server(SDBObject):
         s.save()
         return s
 
-    def __init__(self, id=None, manager=None):
-        SDBObject.__init__(self, id, manager)
+    def __init__(self, id=None, **kw):
+        Model.__init__(self, id, **kw)
         self._reservation = None
         self._instance = None
         self._ssh_client = None
         self._pkey = None
         self._config = None
+        self._ec2 = None
 
-    name = StringProperty()
-    instance_id = StringProperty()
+    name = StringProperty(unique=True, verbose_name="Name")
+    instance_id = StringProperty(verbose_name="Instance ID")
     config_uri = StringProperty()
-    ami_id = StringProperty()
-    zone = StringProperty()
-    security_group = StringProperty()
-    key_name = StringProperty()
-    elastic_ip = StringProperty()
-    instance_type = StringProperty()
-    description = StringProperty()
+    ami_id = StringProperty(verbose_name="AMI ID")
+    zone = StringProperty(verbose_name="Availability Zone")
+    security_group = StringProperty(verbose_name="Security Group", default="default")
+    key_name = StringProperty(verbose_name="Key Name")
+    elastic_ip = StringProperty(verbose_name="Elastic IP")
+    instance_type = StringProperty(verbose_name="Instance Type")
+    description = StringProperty(verbose_name="Description")
+    log = StringProperty()
 
     def setReadOnly(self, value):
         raise AttributeError
@@ -105,7 +113,10 @@ class Server(SDBObject):
     def getInstance(self):
         if not self._instance:
             if self.instance_id:
-                rs = self.ec2.get_all_instances([self.instance_id])
+                try:
+                    rs = self.ec2.get_all_instances([self.instance_id])
+                except:
+                    return None
                 if len(rs) > 0:
                     self._reservation = rs[0]
                     self._instance = self._reservation.instances[0]
@@ -183,6 +194,17 @@ class Server(SDBObject):
     config = property(getConfig, setConfig, None,
                       'The instance data for this server')
 
+    def set_config(self, config):
+        """
+        Set SDB based config
+        """
+        self._config = config
+        self._config.dump_to_sdb("botoConfigs", self.id)
+
+    def load_config(self):
+        self._config = Config(do_load=False)
+        self._config.load_from_sdb("botoConfigs", self.id)
+
     def stop(self):
         if self.instance:
             self.instance.stop()
@@ -193,7 +215,7 @@ class Server(SDBObject):
         ami = ec2.get_all_images(image_ids = [str(self.ami_id)])[0]
         groups = ec2.get_all_security_groups(groupnames=[str(self.security_group)])
         if not self._config:
-            self._config = Config(fp=boto.utils.fetch_file(self.config_uri))
+            self.load_config()
         if not self._config.has_section("Credentials"):
             self._config.add_section("Credentials")
             self._config.set("Credentials", "aws_access_key_id", ec2.aws_access_key_id)
@@ -202,8 +224,8 @@ class Server(SDBObject):
         if not self._config.has_section("Pyami"):
             self._config.add_section("Pyami")
 
-        if self.manager.domain:
-            self._config.set('Pyami', 'server_sdb_domain', self.manager.domain.name)
+        if self._manager.domain:
+            self._config.set('Pyami', 'server_sdb_domain', self._manager.domain.name)
             self._config.set("Pyami", 'server_sdb_name', self.name)
 
         cfg = StringIO.StringIO()
@@ -216,8 +238,9 @@ class Server(SDBObject):
                     instance_type = self.instance_type,
                     placement = self.zone,
                     user_data = cfg)
-        i = r.instances.next()
+        i = r.instances[0]
         self.instance_id = i.id
+        self.put()
         if self.elastic_ip:
             ec2.associate_address(self.instance_id, self.elastic_ip)
 
@@ -259,8 +282,8 @@ class Server(SDBObject):
         sftp_client = ssh_client.open_sftp()
         return sftp_client.listdir(remotepath)
 
-    def shell(self):
-        ssh_client = self.get_ssh_client()
+    def shell(self, key_file=None):
+        ssh_client = self.get_ssh_client(key_file)
         channel = ssh_client.invoke_shell()
         interactive_shell(channel)
 
@@ -281,7 +304,7 @@ class Server(SDBObject):
             sftp_client.remove(BotoConfigPath)
         except:
             pass
-        command = 'ec2-bundle-vol '
+        command = 'sudo ec2-bundle-vol '
         command += '-c %s -k %s ' % (remote_cert_file, remote_key_file)
         command += '-u %s ' % self._reservation.owner_id
         command += '-p %s ' % prefix
@@ -331,6 +354,35 @@ class Server(SDBObject):
         self.image_id = self.ec2.register_image('%s/%s.manifest.xml' % (bucket, prefix))
         return self.image_id
 
+    def attach_volume(self, volume, device="/dev/sdp"):
+        """
+        Attach an EBS volume to this server
+
+        :param volume: EBS Volume to attach
+        :type volume: boto.ec2.volume.Volume
+
+        :param device: Device to attach to (default to /dev/sdp)
+        :type device: string
+        """
+        if hasattr(volume, "id"):
+            volume_id = volume.id
+        else:
+            volume_id = volume
+        return self.ec2.attach_volume(volume_id=volume_id, instance_id=self.instance_id, device=device)
+
+    def detach_volume(self, volume):
+        """
+        Detach an EBS volume from this server
+
+        :param volume: EBS Volume to detach
+        :type volume: boto.ec2.volume.Volume
+        """
+        if hasattr(volume, "id"):
+            volume_id = volume.id
+        else:
+            volume_id = volume
+        return self.ec2.detach_volume(volume_id=volume_id, instance_id=self.instance_id)
+
     def install_package(self, package_name):
         print 'installing %s...' % package_name
         command = 'yum -y install %s' % package_name
@@ -341,5 +393,3 @@ class Server(SDBObject):
         print '\t%s' % response
         print '\t%s' % t[2].read()
         print '...complete!'
-
-    
