@@ -24,10 +24,12 @@ import urllib
 from boto.connection import AWSQueryConnection
 from boto.rds.dbinstance import DBInstance
 from boto.rds.dbsecuritygroup import DBSecurityGroup
+from boto.rds.optiongroup  import OptionGroup, OptionGroupOption
 from boto.rds.parametergroup import ParameterGroup
 from boto.rds.dbsnapshot import DBSnapshot
 from boto.rds.event import Event
 from boto.rds.regioninfo import RDSRegionInfo
+from boto.rds.dbsubnetgroup import DBSubnetGroup
 
 
 def regions():
@@ -82,7 +84,7 @@ class RDSConnection(AWSQueryConnection):
 
     DefaultRegionName = 'us-east-1'
     DefaultRegionEndpoint = 'rds.amazonaws.com'
-    APIVersion = '2012-09-17'
+    APIVersion = '2013-05-15'
 
     def __init__(self, aws_access_key_id=None, aws_secret_access_key=None,
                  is_secure=True, port=None, proxy=None, proxy_port=None,
@@ -103,7 +105,7 @@ class RDSConnection(AWSQueryConnection):
                                     validate_certs=validate_certs)
 
     def _required_auth_capability(self):
-        return ['rds']
+        return ['hmac-v4']
 
     # DB Instance methods
 
@@ -275,10 +277,10 @@ class RDSConnection(AWSQueryConnection):
                         * SQL Server:
                           Not applicable and must be None.
 
-        :type param_group: str
-        :param param_group: Name of DBParameterGroup to associate with
-                            this DBInstance.  If no groups are specified
-                            no parameter groups will be used.
+        :type param_group: str or ParameterGroup object
+        :param param_group: Name of DBParameterGroup or ParameterGroup instance
+                            to associate with this DBInstance.  If no groups are
+                            specified no parameter groups will be used.
 
         :type security_groups: list of str or list of DBSecurityGroup objects
         :param security_groups: List of names of DBSecurityGroup to
@@ -397,7 +399,9 @@ class RDSConnection(AWSQueryConnection):
                   'DBInstanceClass': instance_class,
                   'DBInstanceIdentifier': id,
                   'DBName': db_name,
-                  'DBParameterGroupName': param_group,
+                  'DBParameterGroupName': (param_group.name
+                                           if isinstance(param_group, ParameterGroup)
+                                           else param_group),
                   'DBSubnetGroupName': db_subnet_group_name,
                   'Engine': engine,
                   'EngineVersion': engine_version,
@@ -510,10 +514,10 @@ class RDSConnection(AWSQueryConnection):
         :type id: str
         :param id: Unique identifier for the new instance.
 
-        :type param_group: str
-        :param param_group: Name of DBParameterGroup to associate with
-                            this DBInstance.  If no groups are specified
-                            no parameter groups will be used.
+        :type param_group: str or ParameterGroup object
+        :param param_group: Name of DBParameterGroup or ParameterGroup instance
+                            to associate with this DBInstance.  If no groups are
+                            specified no parameter groups will be used.
 
         :type security_groups: list of str or list of DBSecurityGroup objects
         :param security_groups: List of names of DBSecurityGroup to authorize on
@@ -584,7 +588,9 @@ class RDSConnection(AWSQueryConnection):
         """
         params = {'DBInstanceIdentifier': id}
         if param_group:
-            params['DBParameterGroupName'] = param_group
+            params['DBParameterGroupName'] = (param_group.name
+                                              if isinstance(param_group, ParameterGroup)
+                                              else param_group)
         if security_groups:
             l = []
             for group in security_groups:
@@ -1197,3 +1203,229 @@ class RDSConnection(AWSQueryConnection):
         if marker:
             params['Marker'] = marker
         return self.get_list('DescribeEvents', params, [('Event', Event)])
+
+    def create_db_subnet_group(self, name, desc, subnet_ids):
+        """
+        Create a new Database Subnet Group.
+
+        :type name: string
+        :param name: The identifier for the db_subnet_group
+
+        :type desc: string
+        :param desc: A description of the db_subnet_group
+
+        :type subnet_ids: list
+        :param subnets: A list of the subnet identifiers to include in the
+                        db_subnet_group
+
+        :rtype: :class:`boto.rds.dbsubnetgroup.DBSubnetGroup
+        :return: the created db_subnet_group
+        """
+
+        params = {'DBSubnetGroupName': name,
+                  'DBSubnetGroupDescription': desc}
+        self.build_list_params(params, subnet_ids, 'SubnetIds.member')
+
+        return self.get_object('CreateDBSubnetGroup', params, DBSubnetGroup)
+
+    def delete_db_subnet_group(self, name):
+        """
+        Delete a Database Subnet Group.
+
+        :type name: string
+        :param name: The identifier of the db_subnet_group to delete
+
+        :rtype: :class:`boto.rds.dbsubnetgroup.DBSubnetGroup`
+        :return: The deleted db_subnet_group.
+        """
+
+        params = {'DBSubnetGroupName': name}
+
+        return self.get_object('DeleteDBSubnetGroup', params, DBSubnetGroup)
+
+
+    def get_all_db_subnet_groups(self, name=None, max_records=None, marker=None):
+        """
+        Retrieve all the DBSubnetGroups in your account.
+
+        :type name: str
+        :param name: DBSubnetGroup name If supplied, only information about
+                     this DBSubnetGroup will be returned. Otherwise, info
+                     about all DBSubnetGroups will be returned.
+
+        :type max_records: int
+        :param max_records: The maximum number of records to be returned.
+                            If more results are available, a Token will be
+                            returned in the response that can be used to
+                            retrieve additional records.  Default is 100.
+
+        :type marker: str
+        :param marker: The marker provided by a previous request.
+
+        :rtype: list
+        :return: A list of :class:`boto.rds.dbsubnetgroup.DBSubnetGroup`
+        """
+        params = dict()
+        if name != None:
+            params['DBSubnetGroupName'] = name
+        if max_records != None:
+            params['MaxRecords'] = max_records
+        if marker != None:
+            params['Marker'] = marker
+
+        return self.get_list('DescribeDBSubnetGroups', params, [('DBSubnetGroup',DBSubnetGroup)])
+
+    def modify_db_subnet_group(self, name, description=None, subnet_ids=None):
+        """
+        Modify a parameter group for your account.
+
+        :type name: string
+        :param name: The name of the new parameter group
+
+        :type parameters: list of :class:`boto.rds.parametergroup.Parameter`
+        :param parameters: The new parameters
+
+        :rtype: :class:`boto.rds.parametergroup.ParameterGroup`
+        :return: The newly created ParameterGroup
+        """
+        params = {'DBSubnetGroupName': name}
+        if description != None:
+            params['DBSubnetGroupDescription'] = description
+        if subnet_ids != None:
+            self.build_list_params(params, subnet_ids, 'SubnetIds.member')
+
+        return self.get_object('ModifyDBSubnetGroup', params, DBSubnetGroup)
+
+    def create_option_group(self, name, engine_name, major_engine_version,
+                            description=None):
+        """
+        Create a new option group for your account.
+        This will create the option group within the region you
+        are currently connected to.
+
+        :type name: string
+        :param name: The name of the new option group
+
+        :type engine_name: string
+        :param engine_name: Specifies the name of the engine that this option
+                            group should be associated with.
+
+        :type major_engine_version: string
+        :param major_engine_version: Specifies the major version of the engine
+                                     that this option group should be
+                                     associated with.
+
+        :type description: string
+        :param description: The description of the new option group
+
+        :rtype: :class:`boto.rds.optiongroup.OptionGroup`
+        :return: The newly created OptionGroup
+        """
+        params = {
+            'OptionGroupName': name,
+            'EngineName': engine_name,
+            'MajorEngineVersion': major_engine_version,
+            'OptionGroupDescription': description,
+        }
+        group = self.get_object('CreateOptionGroup', params, OptionGroup)
+        group.name = name
+        group.engine_name = engine_name
+        group.major_engine_version = major_engine_version
+        group.description = description
+        return group
+
+    def delete_option_group(self, name):
+        """
+        Delete an OptionGroup from your account.
+
+        :type key_name: string
+        :param key_name: The name of the OptionGroup to delete
+        """
+        params = {'OptionGroupName': name}
+        return self.get_status('DeleteOptionGroup', params)
+
+    def describe_option_groups(self, name=None, engine_name=None,
+                               major_engine_version=None, max_records=100,
+                               marker=None):
+        """
+        Describes the available option groups.
+
+        :type name: str
+        :param name: The name of the option group to describe. Cannot be
+                     supplied together with engine_name or major_engine_version.
+
+        :type engine_name: str
+        :param engine_name: Filters the list of option groups to only include
+                            groups associated with a specific database engine.
+
+        :type major_engine_version: datetime
+        :param major_engine_version: Filters the list of option groups to only
+                                     include groups associated with a specific
+                                     database engine version. If specified, then
+                                     engine_name must also be specified.
+
+        :type max_records: int
+        :param max_records: The maximum number of records to be returned.
+                            If more results are available, a MoreToken will
+                            be returned in the response that can be used to
+                            retrieve additional records.  Default is 100.
+
+        :type marker: str
+        :param marker: The marker provided by a previous request.
+
+        :rtype: list
+        :return: A list of class:`boto.rds.optiongroup.OptionGroup`
+        """
+        params = {}
+        if name:
+            params['OptionGroupName'] = name
+        elif engine_name and major_engine_version:
+            params['EngineName'] = engine_name
+            params['MajorEngineVersion'] = major_engine_version
+        if max_records:
+            params['MaxRecords'] = int(max_records)
+        if marker:
+            params['Marker'] = marker
+        return self.get_list('DescribeOptionGroups', params, [
+            ('OptionGroup', OptionGroup)
+        ])
+
+    def describe_option_group_options(self, engine_name=None,
+                               major_engine_version=None, max_records=100,
+                               marker=None):
+        """
+        Describes the available option group options.
+
+        :type engine_name: str
+        :param engine_name: Filters the list of option groups to only include
+                            groups associated with a specific database engine.
+
+        :type major_engine_version: datetime
+        :param major_engine_version: Filters the list of option groups to only
+                                     include groups associated with a specific
+                                     database engine version. If specified, then
+                                     engine_name must also be specified.
+
+        :type max_records: int
+        :param max_records: The maximum number of records to be returned.
+                            If more results are available, a MoreToken will
+                            be returned in the response that can be used to
+                            retrieve additional records.  Default is 100.
+
+        :type marker: str
+        :param marker: The marker provided by a previous request.
+
+        :rtype: list
+        :return: A list of class:`boto.rds.optiongroup.Option`
+        """
+        params = {}
+        if engine_name and major_engine_version:
+            params['EngineName'] = engine_name
+            params['MajorEngineVersion'] = major_engine_version
+        if max_records:
+            params['MaxRecords'] = int(max_records)
+        if marker:
+            params['Marker'] = marker
+        return self.get_list('DescribeOptionGroupOptions', params, [
+            ('OptionGroupOptions', OptionGroupOption)
+        ])
