@@ -21,6 +21,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
+from __future__ import with_statement
 import errno
 import mimetypes
 import os
@@ -40,6 +41,8 @@ from boto.s3.keyfile import KeyFile
 from boto.s3.user import User
 from boto import UserAgent
 from boto.utils import compute_md5
+from boto.utils import find_matching_headers
+from boto.utils import merge_headers_by_name
 try:
     from hashlib import md5
 except ImportError:
@@ -145,38 +148,38 @@ class Key(object):
             provider = self.bucket.connection.provider
         return provider
 
-    @property
-    def key(self):
+    def _get_key(self):
         return self.name
 
-    @key.setter
-    def key(self, value):
+    def _set_key(self, value):
         self.name = value
 
-    @property
-    def md5(self):
+    key = property(_get_key, _set_key);
+
+    def _get_md5(self):
         if 'md5' in self.local_hashes and self.local_hashes['md5']:
             return binascii.b2a_hex(self.local_hashes['md5'])
 
-    @md5.setter
-    def md5(self, value):
+    def _set_md5(self, value):
         if value:
             self.local_hashes['md5'] = binascii.a2b_hex(value)
         elif 'md5' in self.local_hashes:
             self.local_hashes.pop('md5', None)
 
-    @property
-    def base64md5(self):
+    md5 = property(_get_md5, _set_md5);
+
+    def _get_base64md5(self):
         if 'md5' in self.local_hashes and self.local_hashes['md5']:
             return binascii.b2a_base64(self.local_hashes['md5']).rstrip('\n')
 
-    @base64md5.setter
-    def base64md5(self, value):
+    def _set_base64md5(self, value):
         if value:
             self.local_hashes['md5'] = binascii.a2b_base64(value)
         elif 'md5' in self.local_hashes:
             del self.local_hashes['md5']
 
+    base64md5 = property(_get_base64md5, _set_base64md5);
+
     def get_md5_from_hexdigest(self, md5_hexdigest):
         """
         A utility function to create the 2-tuple (md5hexdigest, base64md5)
@@ -836,23 +839,31 @@ class Key(object):
             headers = {}
         else:
             headers = headers.copy()
+        # Overwrite user-supplied user-agent.
+        for header in find_matching_headers('User-Agent', headers):
+            del headers[header]
         headers['User-Agent'] = UserAgent
         if self.storage_class != 'STANDARD':
             headers[provider.storage_class_header] = self.storage_class
-        if 'Content-Encoding' in headers:
-            self.content_encoding = headers['Content-Encoding']
-        if 'Content-Language' in headers:
-            self.content_encoding = headers['Content-Language']
-        if 'Content-Type' in headers:
+        if find_matching_headers('Content-Encoding', headers):
+            self.content_encoding = merge_headers_by_name(
+                'Content-Encoding', headers)
+        if find_matching_headers('Content-Language', headers):
+            self.content_language = merge_headers_by_name(
+                'Content-Language', headers)
+        content_type_headers = find_matching_headers('Content-Type', headers)
+        if content_type_headers:
             # Some use cases need to suppress sending of the Content-Type
             # header and depend on the receiving server to set the content
             # type. This can be achieved by setting headers['Content-Type']
             # to None when calling this method.
-            if headers['Content-Type'] is None:
+            if (len(content_type_headers) == 1 and
+                headers[content_type_headers[0]] is None):
                 # Delete null Content-Type value to skip sending that header.
-                del headers['Content-Type']
+                del headers[content_type_headers[0]]
             else:
-                self.content_type = headers['Content-Type']
+                self.content_type = merge_headers_by_name(
+                    'Content-Type', headers)
         elif self.path:
             self.content_type = mimetypes.guess_type(self.path)[0]
             if self.content_type == None:
@@ -1454,7 +1465,7 @@ class Key(object):
                     if i == cb_count or cb_count == -1:
                         cb(data_len, cb_size)
                         i = 0
-        except IOError as e:
+        except IOError, e:
             if e.errno == errno.ENOSPC:
                 raise StorageDataError('Out of space for destination file '
                                        '%s' % fp.name)
